@@ -142,13 +142,19 @@ enum {MAXSHORTBAND = 36};
 static void qlevel(CoderInfo *coderInfo,
                    const double *xr0,
                    const double *bandqual,
-                   int gnum
+                   int gnum,
+                   int pnslevel
                   )
 {
     int sb, cnt;
-    // 1.5dB step
-    static const double sfstep = 20.0 / 1.5 / M_LN10;
+#ifndef __clang__
+    /* 2^0.25 (1.50515 dB) step from AAC specs */
+    static const double sfstep = 1.0 / log10(sqrt(sqrt(2.0)));
+#else
+    static const double sfstep = 20 / 1.50515;
+#endif
     int gsize = coderInfo->groups.len[gnum];
+    double pnsthr = 0.1 * pnslevel;
 #ifdef __SSE2__
     int cpuid[4];
     int sse2 = 0;
@@ -169,42 +175,58 @@ static void qlevel(CoderInfo *coderInfo,
       double sfacfix;
       int sfac;
       double rmsx;
+      double etot;
       int xitab[8 * MAXSHORTBAND];
       int *xi;
       int start, end;
       const double *xr;
       int win;
 
+      if (coderInfo->book[coderInfo->bandcnt] != HCB_NONE)
+      {
+          coderInfo->bandcnt++;
+          continue;
+      }
+
       start = coderInfo->sfb_offset[sb];
       end = coderInfo->sfb_offset[sb+1];
 
-      rmsx = 0.0;
+      etot = 0.0;
       xr = xr0;
       for (win = 0; win < gsize; win++)
       {
           for (cnt = start; cnt < end; cnt++)
           {
               double e = xr[cnt] * xr[cnt];
-              rmsx += e;
+              etot += e;
           }
           xr += BLOCK_LEN_SHORT;
       }
-      rmsx /= ((end - start) * gsize);
-      rmsx = sqrt(rmsx);
+      etot /= (double)gsize;
+      rmsx = sqrt(etot / (end - start));
 
       if ((rmsx < NOISEFLOOR) || (!bandqual[sb]))
       {
-          coderInfo->book[coderInfo->bandcnt] = ZERO_HCB;
-          coderInfo->sf[coderInfo->bandcnt++] = 0;
+          coderInfo->book[coderInfo->bandcnt++] = HCB_ZERO;
           continue;
       }
 
-      //printf("qual:%f/%f\n", bandqual[sb], bandqual[sb]/rmsx);
-      sfac = lrint(log(bandqual[sb] / rmsx) * sfstep);
+#ifndef DRM
+      if (bandqual[sb] < pnsthr)
+      {
+          coderInfo->book[coderInfo->bandcnt] = HCB_PNS;
+          coderInfo->sf[coderInfo->bandcnt] +=
+              lrint(log10(etot) * (0.5 * sfstep));
+          coderInfo->bandcnt++;
+          continue;
+      }
+#endif
+
+      sfac = lrint(log10(bandqual[sb] / rmsx) * sfstep);
       if ((SF_OFFSET - sfac) < 10)
           sfacfix = 0.0;
       else
-          sfacfix = exp(sfac / sfstep);
+          sfacfix = pow(10, sfac / sfstep);
 
       xr = xr0 + start;
       end -= start;
@@ -217,7 +239,6 @@ static void qlevel(CoderInfo *coderInfo,
               for (cnt = 0; cnt < end; cnt += 4)
               {
                   __m128 x = {xr[cnt], xr[cnt + 1], xr[cnt + 2], xr[cnt + 3]};
-                  //printf("%f/%f\n", xr[cnt], xr[cnt] * sfacfix);
 
                   x = _mm_max_ps(x, -x);
                   x *= (__m128){sfacfix, sfacfix, sfacfix, sfacfix};
@@ -226,7 +247,6 @@ static void qlevel(CoderInfo *coderInfo,
                   x += (__m128){MAGIC_NUMBER, MAGIC_NUMBER, MAGIC_NUMBER, MAGIC_NUMBER};
 
                   *(__m128i*)(xi + cnt) = _mm_cvttps_epi32(x);
-                  //printf("%d/%d/%d/%d\n", xi[cnt],xi[cnt+1],xi[cnt+2],xi[cnt+3]);
               }
               for (cnt = 0; cnt < end; cnt++)
               {
@@ -254,7 +274,7 @@ static void qlevel(CoderInfo *coderInfo,
           xr += BLOCK_LEN_SHORT;
       }
       huffbook(coderInfo, xitab, gsize * end);
-      coderInfo->sf[coderInfo->bandcnt++] = SF_OFFSET - sfac;
+      coderInfo->sf[coderInfo->bandcnt++] += SF_OFFSET - sfac;
     }
 }
 
@@ -265,8 +285,6 @@ int BlocQuant(CoderInfo *coder, double *xr, AACQuantCfg *aacquantCfg)
     double *gxr;
 
     coder->global_gain = 0;
-    for (cnt = 0; cnt < coder->sfbn; cnt++)
-        coder->sf[cnt] = 0;
 
     coder->bandcnt = 0;
     coder->datacnt = 0;
@@ -285,7 +303,7 @@ int BlocQuant(CoderInfo *coder, double *xr, AACQuantCfg *aacquantCfg)
         {
             bmask(coder, gxr, bandlvl, cnt,
                   (double)aacquantCfg->quality/DEFQUAL);
-            qlevel(coder, gxr, bandlvl, cnt);
+            qlevel(coder, gxr, bandlvl, cnt, aacquantCfg->pnslevel);
             gxr += coder->groups.len[cnt] * BLOCK_LEN_SHORT;
         }
 
@@ -295,7 +313,7 @@ int BlocQuant(CoderInfo *coder, double *xr, AACQuantCfg *aacquantCfg)
             int book = coder->book[cnt];
             if (!book)
                 continue;
-            if ((book != INTENSITY_HCB) && (book != INTENSITY_HCB2))
+            if ((book != HCB_INTENSITY) && (book != HCB_INTENSITY2))
             {
                 coder->global_gain = coder->sf[cnt];
                 break;
@@ -308,7 +326,7 @@ int BlocQuant(CoderInfo *coder, double *xr, AACQuantCfg *aacquantCfg)
         for (cnt = 0; cnt < coder->bandcnt; cnt++)
         {
             int book = coder->book[cnt];
-            if ((book == INTENSITY_HCB) || (book == INTENSITY_HCB2))
+            if ((book == HCB_INTENSITY) || (book == HCB_INTENSITY2))
             {
                 int diff = coder->sf[cnt] - lastis;
 
@@ -320,7 +338,7 @@ int BlocQuant(CoderInfo *coder, double *xr, AACQuantCfg *aacquantCfg)
                 lastis += diff;
                 coder->sf[cnt] = lastis;
             }
-            else if (book)
+            else if (book != HCB_PNS)
             {
                 int diff = coder->sf[cnt] - lastsf;
 
@@ -338,7 +356,7 @@ int BlocQuant(CoderInfo *coder, double *xr, AACQuantCfg *aacquantCfg)
     return 0;
 }
 
-void BandLimit(unsigned *bw, int rate, SR_INFO *sr, AACQuantCfg *aacquantCfg)
+void CalcBW(unsigned *bw, int rate, SR_INFO *sr, AACQuantCfg *aacquantCfg)
 {
     // find max short frame band
     int max = *bw * (BLOCK_LEN_SHORT << 1) / rate;
@@ -353,7 +371,8 @@ void BandLimit(unsigned *bw, int rate, SR_INFO *sr, AACQuantCfg *aacquantCfg)
         l += sr->cb_width_short[cnt];
     }
     aacquantCfg->max_cbs = cnt;
-    *bw = (double)l * rate / (BLOCK_LEN_SHORT << 1);
+    if (aacquantCfg->pnslevel)
+        *bw = (double)l * rate / (BLOCK_LEN_SHORT << 1);
 
     // find max long frame band
     max = *bw * (BLOCK_LEN_LONG << 1) / rate;
@@ -365,19 +384,25 @@ void BandLimit(unsigned *bw, int rate, SR_INFO *sr, AACQuantCfg *aacquantCfg)
         l += sr->cb_width_long[cnt];
     }
     aacquantCfg->max_cbl = cnt;
+    aacquantCfg->max_l = l;
 
     *bw = (double)l * rate / (BLOCK_LEN_LONG << 1);
 }
 
 enum {MINSFB = 2};
 
-static void calce(double *xr, int *bands, double e[NSFB_SHORT], int maxsfb)
+static void calce(double *xr, int *bands, double e[NSFB_SHORT], int maxsfb,
+                  int maxl)
 {
     int sfb;
+    int l;
+
+    // mute lines above cutoff freq
+    for (l = maxl; l < bands[maxsfb]; l++)
+        xr[l] = 0.0;
+
     for (sfb = MINSFB; sfb < maxsfb; sfb++)
     {
-        int l;
-
         e[sfb] = 0;
         for (l = bands[sfb]; l < bands[sfb + 1]; l++)
             e[sfb] += xr[l] * xr[l];
@@ -397,7 +422,7 @@ static void resete(double min[NSFB_SHORT], double max[NSFB_SHORT],
 static int groups = 0;
 static int frames = 0;
 #endif
-void BlocGroup(double *xr, CoderInfo *coderInfo, int maxsfb)
+void BlocGroup(double *xr, CoderInfo *coderInfo, AACQuantCfg *cfg)
 {
     int win, sfb;
     double e[NSFB_SHORT];
@@ -406,6 +431,7 @@ void BlocGroup(double *xr, CoderInfo *coderInfo, int maxsfb)
     const double thr = 3.0;
     int win0;
     int fastmin;
+    int maxsfb, maxl;
 
     if (coderInfo->block_type != ONLY_SHORT_WINDOW)
     {
@@ -414,6 +440,8 @@ void BlocGroup(double *xr, CoderInfo *coderInfo, int maxsfb)
         return;
     }
 
+    maxl = cfg->max_l / 8;
+    maxsfb = cfg->max_cbs;
     fastmin = ((maxsfb - MINSFB) * 3) >> 2;
 
 #ifdef DRM
@@ -425,7 +453,7 @@ void BlocGroup(double *xr, CoderInfo *coderInfo, int maxsfb)
 #if PRINTSTAT
     frames++;
 #endif
-    calce(xr, coderInfo->sfb_offset, e, maxsfb);
+    calce(xr, coderInfo->sfb_offset, e, maxsfb, maxl);
     resete(min, max, e, maxsfb);
     win0 = 0;
     coderInfo->groups.n = 0;
@@ -433,7 +461,7 @@ void BlocGroup(double *xr, CoderInfo *coderInfo, int maxsfb)
     {
         int fast = 0;
 
-        calce(xr + win * BLOCK_LEN_SHORT, coderInfo->sfb_offset, e, maxsfb);
+        calce(xr + win * BLOCK_LEN_SHORT, coderInfo->sfb_offset, e, maxsfb, maxl);
         for (sfb = MINSFB; sfb < maxsfb; sfb++)
         {
             if (min[sfb] > e[sfb])
